var ImageGallery = (function() {

	var ImageLoader = function() {
		this.imageUrls = [];
		this.images = [];
		this.imageCount = 0;
		this.callback = null;
		this.progressCallback = null;
	};
	
	ImageLoader.prototype.addItem = function(url, data) {
		this.imageUrls[this.imageUrls.length] = { url: url, data: data };
	};
	
	ImageLoader.prototype.load = function(callback, progressCallback) {
		var image;
		this.callback = callback;
		this.progressCallback = progressCallback;
		this.imageCount = this.imageUrls.length;
		for(var i = 0; i < this.imageUrls.length; i++) {
			image = new Image();
			
			image.onload = (function(self, index) { return function() { self.successCallback(index); }; })(this, i);
			image.onerror = (function(self, index) { return function() { self.errorCallback(index); }; })(this, i);
			image.onabort = (function(self, index) { return function() { self.errorCallback(index); }; })(this, i);
			
			image.src = this.imageUrls[i].url;
			this.images[i] = { success: false, image: image, data: this.imageUrls[i].data };
		}
		if(this.progressCallback) {
			this.progressCallback({loaded: this.imageUrls.length - this.imageCount, total: this.imageUrls.length});
		}
	};
	
	ImageLoader.prototype.successCallback = function(index) {
		this.images[index].success = true;
		this.loadCallback();
	};
	
	ImageLoader.prototype.errorCallback = function() {
		this.loadCallback();
	};
	
	ImageLoader.prototype.loadCallback = function() {
		this.imageCount--;
		if(this.progressCallback) {
			this.progressCallback({loaded: this.imageUrls.length - this.imageCount, total: this.imageUrls.length});
		}
		if(this.imageCount === 0) {
			if(this.callback) {
				this.callback(this.images);
			}
		}
	};
	
	var ImageGallery = function(imageContainer, listContainer, images) {
		var loader = new ImageLoader();
		this.imageContainer = (typeof imageContainer === 'string') ? document.getElementById(imageContainer) : imageContainer;
		this.listContainer = (typeof listContainer === 'string') ? document.getElementById(listContainer) : listContainer;
		this.loadingText = document.createTextNode('Loading images...');
		this.images = images;
		this.currentIndex = 0;
		this.menu = [];
		this.imageContainer.appendChild(this.loadingText);
		if(window.location.hash && (window.location.hash.substring(0, 2) === '#i')) {
			var index = parseInt(window.location.hash.substring(2)) - 1;
			if((index >= 0) && (index < (this.images.length))) {
				this.currentIndex = index;
			}
		}
		for(var i = 0; i < this.images.length; i++) {
			loader.addItem(this.images[i], {index: i});
		}
		loader.load(
			(function(self) { return function(images) { self.loaded(images); }; })(this),
			(function(self) { return function(status) { self.loadProgress(status); }; })(this)
		);
	};
	
	ImageGallery.prototype.loaded = function(images) {
		var width = 0;
		var height = 0;
		for(var i = 0; i < images.length; i++) {
			if(images[i].image.width > width) {
				width = images[i].image.width;
			}
			if(images[i].image.height > height) {
				height = images[i].image.height;
			}
			this.menu[i] = document.createElement('a');
			this.menu[i].href = 'javascript:;';
			this.menu[i].className = (i === this.currentIndex) ? 'image-list-item-current' : 'image-list-item';
			this.menu[i].onclick = (function(self, index) {
				return function() { self.show(index); };
			})(this, images[i].data.index);
			this.menu[i].appendChild(document.createTextNode(i + 1));
			this.listContainer.appendChild(this.menu[i]);
			this.listContainer.appendChild(document.createTextNode(' '));
		}
		this.imageContainer.removeChild(this.loadingText);
		this.imageContainer.style.width = width + 'px';
		this.imageContainer.style.height = height + 'px';
		this.imageContainer.style.backgroundImage = 'url(\'' + this.images[this.currentIndex] + '\')';
		this.imageContainer.appendChild(this.img);
	};
	
	ImageGallery.prototype.loadProgress = function(status) {
		this.loadingText.nodeValue = 'Loading images (' + status.loaded + '/' + status.total + ')...';
	};
	
	ImageGallery.prototype.show = function(index) {
		if((index >= 0) && (index < (this.images.length))) {
			this.menu[this.currentIndex].className = 'image-list-item';
			this.menu[index].className = 'image-list-item-current';
			//window.location.href = '#i' + (index + 1);
			this.imageContainer.style.backgroundImage = 'url(\'' + this.images[index] + '\')';
			this.currentIndex = index;
		}
	};
	
	ImageGallery.prototype.prev = function() {
		var index = (this.currentIndex <= 0) ? (this.images.length - 1) : this.currentIndex - 1;
		this.show(index);
	};
	
	ImageGallery.prototype.next = function() {
		var index = (this.currentIndex >= (this.images.length - 1)) ? 0 : this.currentIndex + 1;
		this.show(index);
	};
	
	return ImageGallery;
	
})();