var MultiField = function(jq, name, autocompleteSource, validationSource) {
	this.jq = jq;
	this.formName = name;
	this.validationSource = validationSource;
	this.inputContainer = jq.find('.form-multi-input-container');
	this.input = jq.find('.form-multi-input');
	this.offset = this.input.offset().left;
	this.ruler = $('<div>').addClass('form-multi-ruler').hide().appendTo(document.body);
	this.jq.find('.form-multi-remove').click($.proxy(this, 'removeItem'));
	this.jq.mousedown($.proxy(this, 'clickHandler'));
	this.input.autocomplete({
		source: autocompleteSource,
		select: $.proxy(function(e, ui) {
			e.stopPropagation();
			var value = ui.item.val || ui.item.label;
			if(!this.hasValue(value)) {
				this.addItem(ui.item.html, value);
				return false;
			}
			else {
				this.addError();
				setTimeout($.proxy(this, 'resizeForm'), 10);
			}
		}, this)
	}).keydown($.proxy(this, 'inputHandler'));
};
MultiField.prototype.inputHandler = function(e) {
	if((e.which === 188) || (e.which == 13)) {
		e.preventDefault();
		this.submitForm();
	}
	else
	{
		this.clearError();
		setTimeout($.proxy(this, 'resizeForm'), 10);
	}
};
MultiField.prototype.resizeForm = function() {
	this.ruler.text(this.input.val());
	var overhead = 24 +
		parseInt(this.inputContainer.css('marginLeft')) +
		parseInt(this.inputContainer.css('marginRight')) +
		parseInt(this.inputContainer.css('paddingLeft')) +
		parseInt(this.inputContainer.css('paddingRight'));
	var newWidth = this.ruler.width() + 32;
	var limit = this.jq.innerWidth() - overhead;
	if(newWidth > limit) {
		newWidth = limit;
	}
	this.input.css('width', newWidth + 'px');
	if(this.input.offset().left !== this.offset) {
		this.scroll();
		this.offset = this.input.offset().left;
	}
};
MultiField.prototype.clickHandler = function(e) {
	if((e.target === this.jq.get(0)) || (e.target === this.jq.find('.form-multi-input-container').get(0))) {
		e.preventDefault();
		this.focusForm();
	}
};
MultiField.prototype.focusForm = function() {
	this.scroll();
	this.input.focus();
};
MultiField.prototype.scroll = function() {
	this.jq.prop('scrollTop', this.jq.prop('scrollHeight'));
};
MultiField.prototype.submitForm = function() {
	if(this.input.val().length) {
		$.ajax({
			url: this.validationSource,
			data: {term: this.input.val()},
			success: $.proxy(function(data) {
				if(data && !this.hasValue(data.val)) {
					this.addItem(data.html, data.val);
				}
				else {
					this.addError();
					this.input.autocomplete('close');
				}
			}, this)
		});
	}
};
MultiField.prototype.hasValue = function(value) {
	return !!this.jq.find('input[name="' + this.formName + '"][value="' + value + '"]').length;
}
MultiField.prototype.addItem = function(label, value) {
	this.resetForm();
	$('<li>')
		.addClass('form-multi-item')
		.append(label)
		.append(' &nbsp;')
		.append($('<a>', {href: 'javascript:;', title: 'Remove'})
			.addClass('form-multi-remove')
			.click($.proxy(this, 'removeItem'))
			.text('x'))
		.append($('<input>', {type: 'hidden', name: this.formName}).val(value))
		.insertBefore(this.inputContainer);
	this.focusForm();
};
MultiField.prototype.removeItem = function(e) {
	$(e.currentTarget).parent().remove();
};
MultiField.prototype.resetForm = function() {
	this.clearError();
	this.input.val('').css('width', '32px').autocomplete('close');
	this.offset = this.input.offset().left;
};
MultiField.prototype.addError = function() {
	if(!this.inputContainer.hasClass('form-multi-error')) {
		this.inputContainer.addClass('form-multi-error');
	}
};
MultiField.prototype.clearError = function() {
	this.inputContainer.removeClass('form-multi-error');
};

function imageFormChange(input) {
	if($(input).val() !== '') {
		var jqdiv = $('.form_imageCurrentField');
		jqdiv.find('input').unbind('change');
		jqdiv.append(' - ').append($('<a>').attr('href', 'javascript:;').click(function() { jqdiv.remove(); }).append('Remove'));
		jqdiv.removeClass('form_imageCurrentField');
		$('#form_imageUploadFields').append(
			$('<div>').addClass('form_imageCurrentField').append(
				$('<input>').addClass('input-file').attr({name: 'upload_images[]', type: 'file'}).change(function() { imageFormChange(this); })
			)
		);
	}
}

$(function() {
	new MultiField($('#form_authors'), 'authors[]', '/ajax.php?a=getuser', '/ajax.php?a=getuserlink');
	new MultiField($('#form_tags'), 'tags[]', '/ajax.php?a=listtags', '/ajax.php?a=gettag');

	$('.form_imageCurrentField > input').change(function() {
		imageFormChange(this);
	});
	
	$('.section-form-image').click(function(e) {
		var jq = $(this);
		var box = jq.find('input');
		if(e.target !== box.get(0)) {
			box.prop('checked', !box.prop('checked'));
		}
		if(box.prop('checked')) {
			jq.find('.section-form-image-inner').removeClass('section-form-image-inner-inactive');
		}
		else {
			jq.find('.section-form-image-inner').addClass('section-form-image-inner-inactive');
		}
	});
});