var CuteSpinner = (function() {
	var objectName = 'CuteSpinner';
	
	var object = {
		objectId: 0,
		objectList: [],
		viewportWidth: function() {
			return window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
		},
		viewportHeight: function() {
			return window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
		},
		setRotation: function(element, degrees) {
			var i, properties = [
				'transform',
				'WebkitTransform',
				'MozTransform',
				'msTransform',
				'OTransform'
			];
			for(i = 0; i < properties.length; i++) {
				element.style[properties[i]] = 'rotate(' + degrees + 'deg)';
			}
		},
		spinner: function(sourceElement) {
			this.id = this.parent.objectId++;
			this.parent.objectList[this.id] = this;
			this.sourceElement = (typeof sourceElement == 'string') ? document.getElementById(sourceElement) : sourceElement;
		}
	}
	
	object.spinner.prototype.parent = object;
	
	// Public properties
	object.spinner.prototype.trailLength = 5;
	object.spinner.prototype.rotationSpeed = 10;
	object.spinner.prototype.movementSpeed = 10;
	object.spinner.prototype.framesPerSecond = 12;
	object.spinner.prototype.randomizeStartingPoint = false;
	
	object.spinner.prototype.go = function() {
		var i;
		this.objectList = [];
		this.currentObject = 0;
		if(this.trailLength < 1) {
			this.trailLength = 1;
		}
		for(i = 0; i < this.trailLength; i++) {
			this.objectList[i] = this.sourceElement.cloneNode(true);
			document.body.appendChild(this.objectList[i]);
			this.objectList[i].style.position = 'fixed';
			this.parent.setRotation(this.objectList[i], 0);
			this.objectList[i].style.left = '0px';
			this.objectList[i].style.top = '0px';
		}
		if(this.randomizeStartingPoint) {
			this.rotation = Math.round(Math.random() * 359);
			this.parent.setRotation(this.objectList[this.currentObject], this.rotation);
			this.x = Math.round(Math.random() * (this.parent.viewportWidth() - this.objectList[this.currentObject].offsetWidth - 1));
			this.y = Math.round(Math.random() * (this.parent.viewportHeight() - this.objectList[this.currentObject].offsetHeight - 1));
			for(i = 0; i < this.trailLength; i++) {
				this.parent.setRotation(this.objectList[i], this.rotation);
				this.objectList[i].style.left = this.x + 'px';
				this.objectList[i].style.top = this.y + 'px';
			}
			this.directionX = Math.round(Math.random()) ? 1 : -1;
			this.directionY = Math.round(Math.random()) ? 1 : -1;
		}
		else {
			this.rotation = 0;
			this.x = 0;
			this.y = 0;
			this.directionX = 1;
			this.directionY = 1;
		}
		setInterval((objectName + '.objectList[' + this.id + '].frame()'), (1000 / this.framesPerSecond));
	};
	
	object.spinner.prototype.frame = function() {
		var i, maxX, maxY;
		
		this.currentObject = (this.currentObject + 1) % this.trailLength;
		
		this.rotation = (this.rotation + this.rotationSpeed) % 360;
		this.parent.setRotation(this.objectList[this.currentObject], this.rotation);
		
		maxX = this.parent.viewportWidth() - this.objectList[this.currentObject].offsetWidth - 1;
		this.x += this.directionX * this.movementSpeed;
		if(this.x < 0) {
			this.x -= this.x;
			this.directionX = 1;
		}
		if(this.x > maxX) {
			this.x -= this.x - maxX;
			this.directionX = -1;
		}
		
		maxY = this.parent.viewportHeight() - this.objectList[this.currentObject].offsetHeight - 1;
		this.y += this.directionY * this.movementSpeed;
		if(this.y < 0) {
			this.y -= this.y;
			this.directionY = 1;
		}
		if(this.y > maxY) {
			this.y -= this.y - maxY;
			this.directionY = -1;
		}
		
		this.objectList[this.currentObject].style.left = this.x + 'px';
		this.objectList[this.currentObject].style.top = this.y + 'px';
		
		for(i = 0; i < this.trailLength; i++) {
			index = (this.trailLength + this.currentObject - i) % this.trailLength;
			this.objectList[index].style.zIndex = this.trailLength - i;
			this.objectList[index].style.opacity = 1 - (1 / (this.trailLength + 1)) * i;
		}
	};
	
	return object;
})();